@@ -1057,6 +1057,8 @@ set_close_buttons_sensitivity (GeditTab      *tab,
 				  (state != GEDIT_TAB_STATE_CLOSING) &&
 				  (state != GEDIT_TAB_STATE_SAVING)  &&
 				  (state != GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW) &&
+				  (state != GEDIT_TAB_STATE_PRINTING) &&
+				  (state != GEDIT_TAB_STATE_PRINT_PREVIEWING) &&				  
 				  (state != GEDIT_TAB_STATE_SAVING_ERROR));
 }
 
