@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit.c 6039 2007-12-28 19:52:37Z sfre $
+ * $Id: gedit.c 6259 2008-05-05 09:24:33Z jessevdk $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -480,7 +480,7 @@ main (int argc, char *argv[])
 
 	gedit_debug_message (DEBUG_APP, "Create bacon connection");
 
-	connection = bacon_message_connection_new ("gedit");
+	connection = bacon_message_connection_new ("gedit-svn");
 
 	if (connection != NULL)
 	{
